@@ -1,12 +1,11 @@
 from functools import wraps
 
-import requests
 from urecord import Record
 from urlobject import URLObject, encode_component
 
 from conneg.accept import make_accept_header
 from conneg.default import DEFAULT_REGISTER
-from http import requests_http_backend
+from http import urllib3_http_backend
 from utils.caseless_dict import CaselessDictionary
 
 __all__ = ['API']
@@ -54,7 +53,7 @@ class API(Record('url', 'http')):
 
     """
 
-    def __new__(cls, url, http=requests_http_backend):
+    def __new__(cls, url, http=urllib3_http_backend):
         if not isinstance(url, URLObject):
             url = URLObject.parse(url)
         return super(API, cls).__new__(cls, url, http)
@@ -276,6 +275,8 @@ class API(Record('url', 'http')):
 
             *   A bytestring, with a ``Content-Type`` header (given in
                 ``headers``).
+            *   A :class:`~intessa.conneg.streaming.StreamingBody` (or any
+                file-like object which supports ``__len__()``).
             *   A file-like object, with ``Content-Type`` and
                 ``Content-Length`` headers.
             *   Any other Python object, with the ``type`` parameter provided.
@@ -357,7 +358,7 @@ def encode_request(headers, data, type=None, codec_register=DEFAULT_REGISTER):
 
     :param headers:
         The HTTP headers for the request. A new dictionary of headers will be
-        returned which will contain a ``Content-Type``.
+        returned which will contain a ``Content-Type`` and ``Content-Length``.
 
     :param data:
         A Python object representing the body of the request.
@@ -365,6 +366,8 @@ def encode_request(headers, data, type=None, codec_register=DEFAULT_REGISTER):
         There are several valid values this parameter can take:
 
         *   A bytestring (with a ``Content-Type`` header given in ``headers``).
+        *   A :class:`~intessa.conneg.streaming.StreamingBody` (or any
+            file-like object which supports ``__len__()``).
         *   A file-like object (with ``Content-Type`` and ``Content-Length``
             headers).
         *   Any other Python object, with the ``type`` parameter provided.
@@ -385,15 +388,21 @@ def encode_request(headers, data, type=None, codec_register=DEFAULT_REGISTER):
     """
 
     headers = CaselessDictionary(headers)
-    if not ((isinstance(data, str) and 'content-type' in headers) or
-            ((hasattr(data, 'read') or hasattr(data, '__iter__')) and
-             'content-type' in headers and
-             'content-length' in headers)):
-        if type is None:
+
+    if 'content-type' not in headers:
+        if type is not None:
+            headers['content-type'], data = codec_register.encode(type, data)
+        else:
             raise TypeError(
                 "Request data provided with no content type information")
-        headers['content-type'], data = codec_register.encode(type, data)
 
-    if not headers['content-type']:
-        del headers['content-type']
+    if 'content-length' not in headers:
+        if hasattr(data, 'read'):
+            try:
+                headers['content-length'] = str(len(data))
+            except Exception, exc:
+                raise TypeError("Streaming request provided with no length")
+        else:
+            headers['content-length'] = str(len(data))
+
     return headers, data
